///###////////////////////////////////////////////////////////////////////////
//
// Burton Computer Corporation
// http://www.burton-computer.com
// http://www.cooldevtools.com
// $Id: FrequencyDB.h,v 1.6 2002/09/19 16:15:36 bburton Exp $
//
// Copyright (C) 2000 Burton Computer Corporation
// ALL RIGHTS RESERVED
//
// This program is open source software; you can redistribute it
// and/or modify it under the terms of the Q Public License (QPL)
// version 1.0. Use of this software in whole or in part, including
// linking it (modified or unmodified) into other programs is
// subject to the terms of the QPL.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// Q Public License for more details.
//
// You should have received a copy of the Q Public License
// along with this program; see the file LICENSE.txt.  If not, visit
// the Burton Computer Corporation or CoolDevTools web site
// QPL pages at:
//
//    http://www.burton-computer.com/qpl.html
//    http://www.cooldevtools.com/qpl.html
//

#ifndef _FrequencyDB_h
#define _FrequencyDB_h

#include "Message.h"

class FrequencyDBImpl;

class FrequencyDB
{
public:
  FrequencyDB();
  ~FrequencyDB();

  bool open(const string &filename);
  void close();
  void flush();

  void addWord(const string &word,
               int good_count,
               int spam_count);

  void removeWord(const string &word,
                  int good_count,
                  int spam_count);

  void getWordCounts(const string &word,
                     int &good_count,
                     int &spam_count);

  void getMessageCounts(int &good_message_count,
                        int &spam_message_count);

  bool containsMessage(const Message &msg,
                       bool &is_spam);

  void addMessage(const Message &msg, bool is_spam);

  void removeMessage(const Message &msg);

  bool firstWord(string &word,
                 int &good_count,
                 int &spam_count) const;

  bool nextWord(string &word,
                int &good_count,
                int &spam_count) const;

  string getDatabaseType() const;

  struct WordData
  {
    WordData()
      : good_count(0),
        spam_count(0),
        flags(0)
    {
    }

    WordData(int _good_count,
             int _spam_count)
      : good_count(_good_count),
        spam_count(_spam_count),
        flags(0)
    {
    }

    void clear()
    {
      good_count = 0;
      spam_count = 0;
      flags = 0;
    }

    int good_count;
    int spam_count;
    long flags;
  };

private:
  /// Not implemented.
  FrequencyDB(const FrequencyDB &);

  /// Not implemented.
  FrequencyDB& operator=(const FrequencyDB &);

private:
  void adjustWordCounts(const string &word,
                        int delta,
                        bool already_exists,
                        bool is_spam);

private:
  NewPtr<FrequencyDBImpl> m_db;
};

#endif // _FrequencyDB_h
