///###////////////////////////////////////////////////////////////////////////
//
// Burton Computer Corporation
// http://www.burton-computer.com
// http://www.cooldevtools.com
// $Id: FrequencyDBImpl_cache.cc,v 1.3 2002/09/10 22:48:44 bburton Exp $
//
// Copyright (C) 2000 Burton Computer Corporation
// ALL RIGHTS RESERVED
//
// This program is open source software; you can redistribute it
// and/or modify it under the terms of the Q Public License (QPL)
// version 1.0. Use of this software in whole or in part, including
// linking it (modified or unmodified) into other programs is
// subject to the terms of the QPL.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// Q Public License for more details.
//
// You should have received a copy of the Q Public License
// along with this program; see the file LICENSE.txt.  If not, visit
// the Burton Computer Corporation or CoolDevTools web site
// QPL pages at:
//
//    http://www.burton-computer.com/qpl.html
//    http://www.cooldevtools.com/qpl.html
//

#include <unistd.h>
#include <fcntl.h>
#include "FrequencyDBImpl_cache.h"

static const string DATABASE_TYPE("-cached");

FrequencyDBImpl_cache::FrequencyDBImpl_cache(FrequencyDBImpl *db)
  : m_db(db)
{
  assert(db);
}

FrequencyDBImpl_cache::~FrequencyDBImpl_cache()
{
  close();
}

bool FrequencyDBImpl_cache::open(const string &filename)
{
  close();
  return m_db->open(filename);
}

void FrequencyDBImpl_cache::close()
{
  flush();
  m_db->close();
}

void FrequencyDBImpl_cache::flush()
{
  for (map_iter_t i = m_cache.begin(); i != m_cache.end(); ++i) {
    if (i->second.is_dirty) {
      m_db->writeWord(i->first, *i->second.counts);
    }
    delete i->second.counts;
  }
  m_cache.clear();
  m_db->flush();
}

void FrequencyDBImpl_cache::writeWord(const string &word,
                                      const FrequencyDB::WordData &counts)
{
  map_iter_t i = m_cache.find(word);
  if (i == m_cache.end()) {
    m_cache.insert(make_pair(word, CacheEntry(true, new FrequencyDB::WordData(counts))));
  } else {
    i->second.is_dirty = true;
    *i->second.counts = counts;
  }
}

bool FrequencyDBImpl_cache::readWord(const string &word,
                                     FrequencyDB::WordData &counts)
{
  map_const_iter_t i = m_cache.find(word);
  if (i != m_cache.end()) {
    counts = *i->second.counts;
    return true;
  }

  if (m_db->readWord(word, counts)) {
    m_cache.insert(make_pair(word, CacheEntry(false, new FrequencyDB::WordData(counts))));
    return true;
  }

  return false;
}

bool FrequencyDBImpl_cache::firstWord(string &word,
                                      FrequencyDB::WordData &counts)
{
  return m_db->firstWord(word, counts);
}

bool FrequencyDBImpl_cache::nextWord(string &word,
                                     FrequencyDB::WordData &counts)
{
  return m_db->nextWord(word, counts);
}

string FrequencyDBImpl_cache::getDatabaseType() const
{
  return m_db->getDatabaseType() + DATABASE_TYPE;
}
