///###////////////////////////////////////////////////////////////////////////
//
// Burton Computer Corporation
// http://www.burton-computer.com
// http://www.cooldevtools.com
// $Id: LockFile.cc,v 1.1.1.1 2002/08/28 19:41:19 bburton Exp $
//
// Copyright (C) 2000 Burton Computer Corporation
// ALL RIGHTS RESERVED
//
// This program is open source software; you can redistribute it
// and/or modify it under the terms of the Q Public License (QPL)
// version 1.0. Use of this software in whole or in part, including
// linking it (modified or unmodified) into other programs is
// subject to the terms of the QPL.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// Q Public License for more details.
//
// You should have received a copy of the Q Public License
// along with this program; see the file LICENSE.txt.  If not, visit
// the Burton Computer Corporation or CoolDevTools web site
// QPL pages at:
//
//    http://www.burton-computer.com/qpl.html
//    http://www.cooldevtools.com/qpl.html
//

#include <string.h>
#include <errno.h>
#include <unistd.h>
#include <fcntl.h>
#include <stdexcept>
#include "LockFile.h"

LockFile::LockFile(const File &file)
  : m_file(file),
    m_lockFD(-1)
{
}

LockFile::~LockFile()
{
  try {
    release();
  } catch (...) {
  }
}

void LockFile::lock(LockType_t lock_type)
{
  release();
  int open_flag = O_CREAT | ((lock_type == SHARED) ? O_RDONLY : O_RDWR);
  m_lockFD = ::open(m_file, open_flag, 0600);
  if (m_lockFD < 0) {
    throw runtime_error(strerror(errno));
  }

  struct flock buf;
  buf.l_type = (lock_type == SHARED) ? F_RDLCK : F_WRLCK;
  buf.l_whence = SEEK_SET;
  buf.l_start = 0;
  buf.l_len = 0;

  int rc = fcntl(m_lockFD, F_SETLKW, &buf);
  if (rc < 0) {
    int saved_errno = errno;
    close(m_lockFD);
    m_lockFD = -1;
    throw runtime_error(strerror(saved_errno));
  }
}

void LockFile::release()
{
  if (m_lockFD >= 0) {
    close(m_lockFD);
    m_lockFD = -1;
  }
}
