///###////////////////////////////////////////////////////////////////////////
//
// Burton Computer Corporation
// http://www.burton-computer.com
// http://www.cooldevtools.com
// $Id: MessageFactory.h,v 1.10 2002/09/17 21:39:37 bburton Exp $
//
// Copyright (C) 2000 Burton Computer Corporation
// ALL RIGHTS RESERVED
//
// This program is open source software; you can redistribute it
// and/or modify it under the terms of the Q Public License (QPL)
// version 1.0. Use of this software in whole or in part, including
// linking it (modified or unmodified) into other programs is
// subject to the terms of the QPL.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// Q Public License for more details.
//
// You should have received a copy of the Q Public License
// along with this program; see the file LICENSE.txt.  If not, visit
// the Burton Computer Corporation or CoolDevTools web site
// QPL pages at:
//
//    http://www.burton-computer.com/qpl.html
//    http://www.cooldevtools.com/qpl.html
//

#ifndef _MessageFactory_h
#define _MessageFactory_h

#include "AbstractMessageFactory.h"

class MessageFactory : public AbstractMessageFactory
{
public:
  MessageFactory();
  ~MessageFactory();

  void initMessage(Message &msg,
                   MimeMessageReader &reader);

  void setReplaceNonAsciiChars(int value)
  {
    if (value >= 0) {
      m_replaceNonAsciiChars = true;
      m_nonAsciiChar = tolower((char)value);
    } else {
      m_replaceNonAsciiChars = false;
    }
  }

  string::size_type getMinWordLength() const
  {
    return m_minWordLength;
  }

  string::size_type getMaxWordLength() const
  {
    return m_maxWordLength;
  }

  bool getRemoveHTML() const
  {
    return m_removeHTML;
  }

  void setMinWordLength(string::size_type value)
  {
    m_minWordLength = value;
  }

  void setMaxWordLength(string::size_type value)
  {
    m_maxWordLength = value;
  }

  void setRemoveHTML(bool value)
  {
    m_removeHTML = value;
  }

  void setUseWordPairs(bool value)
  {
    m_useWordPairs = value;
  }

  enum HeaderSpec_t {
    NORMAL_HEADERS,
    ALL_HEADERS,
    NO_X_HEADERS,
    NO_HEADERS
  };
  void setHeadersToInclude(HeaderSpec_t value)
  {
    m_headersToInclude = value;
  }

private:
  void addHeaderToMessage(const string &name,
                          Message &msg,
                          MimeMessageReader &reader);
  void addWordToMessage(const string &value,
                        Message &msg);
  void addWordPartsToMessage(const string &value,
                             Message &msg);
  void addStringToMessage(const string &value,
                          Message &msg);
  void addTextToMessage(const string &value,
                        Message &msg);
  bool isHTML(const string &lower);
  void addHrefFromTag(const string &tag,
                      Message &msg);
  void removeHTMLFromText(string &text,
                          Message &msg);
  void expandEntitiesInHTML(string &text);
  void addHeadersToMessage(Message &msg,
                           MimeMessageReader &reader);

private:
  string::size_type m_minWordLength;
  string::size_type m_maxWordLength;
  string m_prevWord;

  bool m_replaceNonAsciiChars;
  char m_nonAsciiChar;

  bool m_removeHTML;
  bool m_useWordPairs;
  HeaderSpec_t m_headersToInclude;
};

#endif // _MessageFactory_h
