///###////////////////////////////////////////////////////////////////////////
//
// Burton Computer Corporation
// http://www.burton-computer.com
// http://www.cooldevtools.com
// $Id: SpamFilter.h,v 1.6 2002/09/16 21:20:04 bburton Exp $
//
// Copyright (C) 2000 Burton Computer Corporation
// ALL RIGHTS RESERVED
//
// This program is open source software; you can redistribute it
// and/or modify it under the terms of the Q Public License (QPL)
// version 1.0. Use of this software in whole or in part, including
// linking it (modified or unmodified) into other programs is
// subject to the terms of the QPL.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// Q Public License for more details.
//
// You should have received a copy of the Q Public License
// along with this program; see the file LICENSE.txt.  If not, visit
// the Burton Computer Corporation or CoolDevTools web site
// QPL pages at:
//
//    http://www.burton-computer.com/qpl.html
//    http://www.cooldevtools.com/qpl.html
//

#ifndef _SpamFilter_h
#define _SpamFilter_h

#include "File.h"
#include "FrequencyDB.h"

class SpamFilter
{
public:
  SpamFilter();
  ~SpamFilter();

  void open(const string &db_filename)
  {
    m_db.open(db_filename);
  }

  void open(const File &db_file);

  void close()
  {
    m_db.close();
  }

  void flush()
  {
    m_db.flush();
  }

  FrequencyDB *getDB()
  {
    return &m_db;
  }

  bool isNewMessage(const Message &msg)
  {
    bool is_spam;
    return !m_db.containsMessage(msg, is_spam);
  }

  void ensureGoodMessage(const Message &msg)
  {
    m_db.addMessage(msg, false);
  }

  void ensureSpamMessage(const Message &msg)
  {
    m_db.addMessage(msg, true);
  }

  void removeMessage(const Message &msg)
  {
    m_db.removeMessage(msg);
  }

  double scoreMessage(Message &msg);

  void setGoodBias(int value)
  {
    m_goodBias = value;
  }

  void setTermsForScore(int value)
  {
    m_termsForScore = value;
  }

  void setMaxWordRepeats(int value)
  {
    m_maxWordRepeats = value;
  }

  void setMinWordCount(int value)
  {
    m_minWordCount = value;
  }

  void setNewWordScore(double value)
  {
    m_newWordScore = value;
  }

private:
  double computeRatio(int count,
                      int total_count);
  double scoreToken(Token *tok,
                    int min_count,
                    int good_message_count,
                    int spam_message_count);
  void getSortedTokens(const Message &msg,
                       int max_tokens,
                       vector<Token *> &tokens);

private:
  /// Not implemented.
  SpamFilter(const SpamFilter &);

  /// Not implemented.
  SpamFilter& operator=(const SpamFilter &);

private:
  FrequencyDB m_db;

  int m_goodBias;
  int m_termsForScore;
  int m_maxWordRepeats;
  int m_minWordCount;
  double m_newWordScore;
};

#endif // _SpamFilter_h
